@@ -41,9 +41,9 @@ struct Plugin::EntryPoint::Loader::Priv {
     UString plugin_path ;
 };//end struct Plugin::EntryPoint::Loader::Priv
 
-Plugin::EntryPoint::Loader::Loader (const UString &a_plugin_path)
+Plugin::EntryPoint::Loader::Loader (const UString &a_plugin_path) :
+    m_priv (new Plugin::EntryPoint::Loader::Priv)
 {
-    m_priv = new Plugin::EntryPoint::Loader::Priv ;
     THROW_IF_FAIL (m_priv) ;
 
     config_search_paths ().clear () ;
@@ -56,6 +56,7 @@ Plugin::EntryPoint::Loader::Loader (const UString &a_plugin_path)
 
 Plugin::EntryPoint::Loader::~Loader ()
 {
+    LOG_D ("delete", "destructor-domain") ;
 }
 
 const UString&
@@ -75,7 +76,8 @@ struct Plugin::EntryPoint::Priv {
 
     Priv () :
         is_activated (false)
-    {}
+    {
+    }
 };//end Plugin::EntryPoint::Priv
 
 Plugin::EntryPoint::LoaderSafePtr
@@ -88,16 +90,25 @@ void
 Plugin::EntryPoint::plugin_entry_point_loader
                         (Plugin::EntryPoint::LoaderSafePtr &a_loader)
 {
+    THROW_IF_FAIL (m_priv) ;
     m_priv->entry_point_loader = a_loader ;
 }
 
-Plugin::EntryPoint::EntryPoint ()
+Plugin::EntryPoint::EntryPoint (DynamicModuleSafePtr &a_module) :
+    DynModIface (a_module),
+    m_priv (new Plugin::EntryPoint::Priv)
+{
+}
+
+Plugin::EntryPoint::EntryPoint (DynamicModule *a_module) :
+    DynModIface (a_module),
+    m_priv (new Plugin::EntryPoint::Priv)
 {
-    m_priv = new Plugin::EntryPoint::Priv ;
 }
 
 Plugin::EntryPoint::~EntryPoint ()
 {
+    LOG_D ("delete", "destructor-domain") ;
 }
 
 void
@@ -105,12 +116,14 @@ Plugin::EntryPoint::activate (bool a_activate,
                               ObjectSafePtr &a_ctxt)
 {
     if (a_ctxt) {}
+    THROW_IF_FAIL (m_priv) ;
     m_priv->is_activated = a_activate ;
 }
 
 bool
 Plugin::EntryPoint::is_activated ()
 {
+    THROW_IF_FAIL (m_priv) ;
     return m_priv->is_activated ;
 }
 
@@ -143,12 +156,14 @@ Plugin::EntryPoint::build_absolute_resource_path
 Plugin::DescriptorSafePtr
 Plugin::EntryPoint::descriptor ()
 {
+    THROW_IF_FAIL (m_priv) ;
     return m_priv->descriptor ;
 }
 
 void
 Plugin::EntryPoint::descriptor (Plugin::DescriptorSafePtr &a_desc)
 {
+    THROW_IF_FAIL (m_priv) ;
     m_priv->descriptor = a_desc ;
 }
 
@@ -181,34 +196,42 @@ Plugin::load_entry_point ()
 
     try {
         EntryPoint::LoaderSafePtr loader
-                    (new EntryPoint::Loader (m_priv->descriptor->plugin_path ())) ;
+                (new EntryPoint::Loader (m_priv->descriptor->plugin_path ()));
         m_priv->entry_point =
-            m_priv->module_manager.load<Plugin::EntryPoint>
-                                    (m_priv->descriptor->entry_point_name (),
-                                     *loader) ;
+            m_priv->module_manager.load_iface<Plugin::EntryPoint>
+                                (m_priv->descriptor->entry_point_module_name (),
+                                 m_priv->descriptor->entry_point_interface_name(),
+                                 *loader) ;
+        THROW_IF_FAIL (m_priv->entry_point) ;
+        LOG_REF_COUNT (m_priv->entry_point,
+                       m_priv->descriptor->entry_point_interface_name ()) ;
+
+        LOG_REF_COUNT (loader, "plugin-entry-point-loader") ;
         m_priv->entry_point->plugin_entry_point_loader (loader) ;
+        LOG_REF_COUNT (loader, "plugin-loader") ;
+
         m_priv->entry_point->descriptor (m_priv->descriptor) ;
     } catch (...) {
         THROW ("failed to load plugin entry point '"
-               + m_priv->descriptor->entry_point_name ()
+               + m_priv->descriptor->entry_point_module_name ()
                + " for plugin '" + m_priv->descriptor->name () + "'") ;
     }
 }
 
 Plugin::Plugin (Plugin::DescriptorSafePtr &a_desc,
-                DynamicModuleManager &a_manager)
+                DynamicModuleManager &a_manager) :
+    m_priv (new Plugin::Priv (a_desc, a_manager))
 {
+    THROW_IF_FAIL (m_priv) ;
     THROW_IF_FAIL (a_desc) ;
     THROW_IF_FAIL (Glib::file_test (a_desc->plugin_path (),
                                     Glib::FILE_TEST_IS_DIR)) ;
-
-    m_priv = new Plugin::Priv (a_desc, a_manager);
-
     load_entry_point () ;
 }
 
 Plugin::~Plugin ()
 {
+    LOG_D ("delete", "destructor-domain") ;
 }
 
 Plugin::DescriptorSafePtr
@@ -221,6 +244,7 @@ Plugin::descriptor ()
 void
 Plugin::descriptor (const Plugin::DescriptorSafePtr &a_desc)
 {
+    THROW_IF_FAIL (m_priv) ;
     m_priv->descriptor = a_desc ;
 }
 
@@ -270,14 +294,15 @@ struct PluginManager::Priv {
     {}
 };//end struct PluginManagerPriv
 
-PluginManager::PluginManager (DynamicModuleManager &a_in)
+PluginManager::PluginManager (DynamicModuleManager &a_in) :
+    m_priv (new PluginManager::Priv (a_in))
 {
-    m_priv = new PluginManager::Priv (a_in) ;
     plugins_search_path ().push_back (env::get_system_plugins_dir ()) ;
 }
 
 PluginManager::~PluginManager ()
 {
+    LOG_D ("delete", "destructor-domain") ;
 }
 
 UString
@@ -311,10 +336,13 @@ PluginManager::parse_descriptor (const UString &a_path,
     }
 
 
-    libxmlutils::XMLTextReaderSafePtr reader ;
-    libxmlutils::XMLCharSafePtr xml_str ;
+    using libxmlutils::XMLTextReaderSafePtr ;
+    using libxmlutils::XMLCharSafePtr ;
 
-    reader = xmlNewTextReaderFilename (a_path.c_str ()) ;
+    XMLTextReaderSafePtr reader ;
+    XMLCharSafePtr xml_str ;
+
+    reader.reset (xmlNewTextReaderFilename (a_path.c_str ())) ;
     if (!reader) {
         LOG_ERROR ("could not create xml reader") ;
         return false ;
@@ -329,8 +357,8 @@ PluginManager::parse_descriptor (const UString &a_path,
     if (!goto_next_element_node_and_check (reader, "plugindescriptor")) {
         THROW ("first element node should be 'plugindescriptor'") ;
     }
-    xml_str = xmlTextReaderGetAttribute (reader.get (),
-                                         (const xmlChar*)"autoactivate");
+    xml_str.reset (xmlTextReaderGetAttribute (reader.get (),
+                                             (const xmlChar*)"autoactivate")) ;
     UString autoactivate = xml_str.get () ;
 
     if (autoactivate == "yes") {
@@ -341,8 +369,8 @@ PluginManager::parse_descriptor (const UString &a_path,
 
     desc->can_deactivate (true) ;
     if (desc->auto_activate ()) {
-        xml_str = xmlTextReaderGetAttribute (reader.get (),
-                                             (const xmlChar*) "candeactivate");
+        xml_str.reset (xmlTextReaderGetAttribute (reader.get (),
+                                                  (const xmlChar*) "candeactivate"));
         UString candeactivate = xml_str.get () ;
         if (candeactivate == "no") {
             desc->can_deactivate (false) ;
@@ -354,7 +382,8 @@ PluginManager::parse_descriptor (const UString &a_path,
                + UString (xmlTextReaderConstName (reader.get ())));
     }
 
-    xml_str = xmlTextReaderReadString (reader.get ()) ;
+    xml_str.reset (xmlTextReaderReadString (reader.get ())) ;
+
     desc->name (xml_str.get ()) ;
 
     if (!goto_next_element_node_and_check (reader, "version")) {
@@ -362,7 +391,7 @@ PluginManager::parse_descriptor (const UString &a_path,
                + UString (xmlTextReaderConstName (reader.get ())));
     }
 
-    xml_str = xmlTextReaderReadString (reader.get ());
+    xml_str.reset (xmlTextReaderReadString (reader.get ()));
     desc->version (xml_str.get ()) ;
 
     if (!goto_next_element_node_and_check (reader, "entrypoint")) {
@@ -370,8 +399,21 @@ PluginManager::parse_descriptor (const UString &a_path,
                + UString (xmlTextReaderConstName (reader.get ())));
     }
 
-    xml_str = xmlTextReaderReadString (reader.get ());
-    desc->entry_point_name (xml_str.get ()) ;
+    if (!goto_next_element_node_and_check (reader, "modulename")) {
+        THROW ("expected element 'modulename', got: "
+               + UString (xmlTextReaderConstName (reader.get ()))) ;
+    }
+
+    xml_str.reset (xmlTextReaderReadString (reader.get ()));
+    desc->entry_point_module_name (xml_str.get ()) ;
+
+    if (!goto_next_element_node_and_check (reader, "interfacename")) {
+        THROW ("expected element 'interfacename', got: "
+               + UString (xmlTextReaderConstName (reader.get ()))) ;
+    }
+
+    xml_str.reset (xmlTextReaderReadString (reader.get ()));
+    desc->entry_point_interface_name (xml_str.get ()) ;
 
     if (!goto_next_element_node_and_check (reader, "dependencies")) {
         THROW ("expected element 'dependencies', got: "
@@ -387,13 +429,13 @@ PluginManager::parse_descriptor (const UString &a_path,
             THROW ("expected element 'name', got: "
                    + UString (xmlTextReaderConstName (reader.get ())));
         }
-        xml_str = xmlTextReaderReadString (reader.get ()) ;
+        xml_str.reset (xmlTextReaderReadString (reader.get ())) ;
         name = xml_str.get () ;
         if (!goto_next_element_node_and_check (reader, "version")) {
             THROW ("expected element 'version', got: "
                    + UString (xmlTextReaderConstName (reader.get ())));
         }
-        xml_str = xmlTextReaderReadString (reader.get ()) ;
+        xml_str.reset (xmlTextReaderReadString (reader.get ())) ;
         desc->dependencies ()[name] = xml_str.get () ;
     }
 
@@ -439,7 +481,7 @@ PluginManager::load_dependant_descriptors
                                 (const Plugin::Descriptor &a_desc,
                                  vector<Plugin::DescriptorSafePtr> &a_descs)
 {
-    bool result (false) ;
+    bool result (true) ;
     map<UString, UString>::const_iterator cur_dep ;
     Plugin::DescriptorSafePtr desc ;
     for (cur_dep = a_desc.dependencies ().begin () ;
@@ -530,6 +572,7 @@ PluginManager::load_plugin_from_path (const UString &a_plugin_path,
         return result ;
     }
 
+    UString ppath ;
     vector<Plugin::DescriptorSafePtr>::iterator cur_desc;
     vector<PluginSafePtr> dependances ;
     PluginSafePtr plugin ;
@@ -537,7 +580,14 @@ PluginManager::load_plugin_from_path (const UString &a_plugin_path,
          cur_desc != dependant_descs.end ();
          ++cur_desc) {
         try {
-            plugin = new Plugin (*cur_desc, m_priv->module_manager) ;
+            THROW_IF_FAIL (*cur_desc) ;
+            ppath = (*cur_desc)->plugin_path () ;
+            if (plugins_map ().find (ppath) != plugins_map ().end ()) {
+                plugin = plugins_map ()[ppath] ;
+            } else {
+                plugin.reset (new Plugin (*cur_desc, m_priv->module_manager)) ;
+                plugins_map ()[ppath] = plugin ;
+            }
         } catch (...) {
             LOG_ERROR ("Failed to load dependant plugin '"
                        + (*cur_desc)->name () + "'") ;
@@ -547,7 +597,13 @@ PluginManager::load_plugin_from_path (const UString &a_plugin_path,
     }
 
     try {
-        plugin = new Plugin (descriptor, m_priv->module_manager) ;
+        ppath = descriptor->plugin_path () ;
+        if (plugins_map ().find (ppath) != plugins_map ().end ()) {
+            plugin = plugins_map ()[ppath] ;
+        } else {
+            plugin.reset (new Plugin (descriptor, m_priv->module_manager)) ;
+            plugins_map ()[ppath] = plugin ;
+        }
     } catch (...) {
         LOG_ERROR ("failed to load plugin '" + descriptor->name () + "'") ;
         return result ;
@@ -555,12 +611,21 @@ PluginManager::load_plugin_from_path (const UString &a_plugin_path,
 
     a_deps = dependances ;
     result = plugin ;
+    if (result) {
+        LOG_D ("plugin '"
+                << a_plugin_path
+                << "' refcount: "
+                << (int) result->get_refcount (),
+                "refcount-domain") ;
+    }
+    LOG_D ("loaded plugin from path " << Glib::locale_from_utf8 (a_plugin_path),
+            "plugin-loading-domain") ;
     return result ;
 }
 
 PluginSafePtr
 PluginManager::load_plugin_from_name (const UString &a_name,
-                                        vector<PluginSafePtr> &a_deps)
+                                      vector<PluginSafePtr> &a_deps)
 {
     PluginSafePtr result;
 
@@ -583,8 +648,17 @@ PluginManager::load_plugin_from_name (const UString &a_name,
                                             a_deps) ;
         } catch (...) {}
 
-        if (result) {break;}
+        if (result) {
+            LOG_D ("plugin '"
+                    << a_name
+                    << "' refcount: "
+                    << (int) result->get_refcount (),
+                    "refcount-domain") ;
+            break;
+        }
     }
+    LOG_D ("loaded plugin " << Glib::locale_from_utf8 (a_name),
+           "plugin-loading-domain") ;
     return result ;
 }
 
@@ -603,22 +677,26 @@ PluginManager::load_plugins ()
     vector<UString>::const_iterator cur_dir ;
     string path ;
     for (cur_dir = plugins_search_path ().begin ();
-            cur_dir != plugins_search_path ().end ();
-            ++cur_dir) {
+         cur_dir != plugins_search_path ().end ();
+         ++cur_dir) {
         try {
             Glib::Dir opened_dir (*cur_dir) ;
             for (Glib::DirIterator it = opened_dir.begin ();
-                    it != opened_dir.end ();
-                    ++it) {
+                 it != opened_dir.end ();
+                 ++it) {
                 path = Glib::build_filename (*cur_dir, *it) ;
-                if (plugins_map ().find (Glib::locale_to_utf8 (*it))
-                        != plugins_map ().end ()) {
+                if (plugins_map ().find (Glib::locale_to_utf8 (path))
+                    != plugins_map ().end ()) {
                     continue;
                 }
                 plugin = load_plugin_from_path (Glib::locale_to_utf8 (path),
-                        dependances) ;
+                                                dependances) ;
                 if (plugin) {
-                    plugins_map ()[Glib::locale_to_utf8 (*it)] = plugin ;
+                    LOG_D ("plugin '"
+                            << path
+                            << "' put in  map. Refcount: "
+                            << (int) plugin->get_refcount (),
+                            "refcount-domain") ;
                 }
             }
         } catch (...) {
